<?php

if (!defined('_VALID_MOS') && !defined('_JEXEC'))
    die('Direct Access to ' . basename(__FILE__) . ' is not allowed.');

class PaykeeperCommon {
    private $server = '';
    private $secret = '';

    private $fiscal_cart = array(); //fz54 cart
    private $order_total = 0; //order total sum
    private $shipping_price = 0; //shipping price
    private $use_taxes = false;
    private $use_delivery = false;
    private $delivery_included = false;
    private $delivery_index = -1;
    public $single_item_index = -1;
    public $more_then_one_item_index = -1;
    private $order_params = NULL;
    private $discounts = array();


    public function setOrderParams($order_total = 0, $clientid="", $orderid="", $client_email="", 
                                    $client_phone="", $service_name="", $form_url="", $secret_key="")
    {
       $this->setOrderTotal($order_total);
       $this->order_params = array(
           "sum" => $order_total,
           "clientid" => $clientid,
           "orderid" => $orderid,
           "client_email" => $client_email,
           "client_phone" => $client_phone,
           "phone" => $client_phone,
           "service_name" => $service_name,
           "form_url" => $form_url,
           "secret_key" => $secret_key,
       );
    }

    public function getOrderParams($value)
    {
        return array_key_exists($value, $this->order_params) ? $this->order_params["$value"] : False;
    }

    public function updateFiscalCart($ftype, $name="", $price=0, $quantity=0, $sum=0, $tax="none", $tax_sum=0)
    {
        //update fz54 cart
        if ($ftype === "create") {
            $name = str_replace("\n ", "", $name);
            $name = str_replace("\r ", "", $name);
        }
        $this->fiscal_cart[] = array(
            "name" => $name,
            "price" => $price,
            "quantity" => $quantity,
            "sum" => $sum,
            "tax" => $tax,
            "tax_sum" => number_format($tax_sum, 2, ".", "")
        );
    }

    public function getFiscalCart()
    {
        return $this->fiscal_cart;
    }

    public function setDiscounts($cart_sum, $discount_enabled_flag, $extra_charge_enabled_flag = false)
    {
        $cart_sum_with_discount = $cart_sum;

        if ($discount_enabled_flag) { //set discounts
            $discount_modifier_value = ($this->getOrderTotal() - $this->getShippingPrice())/$cart_sum;

            if ($discount_modifier_value < 1) {
               $this->recalculateFiscalCart($discount_modifier_value, true); 
            }
        }
        else if ($extra_charge_enabled_flag) { //set extra charge or discount in module settings

            $discount_modifier_value = $this->getOrderTotal()/$cart_sum;

            if ($discount_modifier_value > 0) {
               $this->recalculateFiscalCart($discount_modifier_value, false); 
            }
        }
    }

    public function correctPrecision()
    {
        //handle possible precision problem
        $fiscal_cart_sum = $this->getFiscalCartSum(true);
        $total_sum = $this->getOrderTotal(true);

        //debug_info
        //echo "total: " . $total_sum . " - cart: " . $cart_sum;
        $diff_sum = $fiscal_cart_sum - $total_sum;
        if (abs($diff_sum) <= 0.01) {
            $this->setOrderTotal(number_format($total_sum+$diff_sum, 2, ".", ""));
        }
        else {
            if ($this->getUseDelivery() && ($fiscal_cart_sum < $total_sum)) {
                $delivery_pos = count($this->getFiscalCart())-1;
                $this->fiscal_cart[$delivery_pos]["price"] = number_format(
                                   $this->fiscal_cart[$delivery_pos]["price"]+abs($diff_sum), 2, ".", "");
                $this->fiscal_cart[$delivery_pos]["sum"] = number_format(
                                   $this->fiscal_cart[$delivery_pos]["sum"]+abs($diff_sum), 2, ".", "");
                $this->recalculateTaxes($delivery_pos);
            }
        }
    }

    public function setOrderTotal($value)
    {
        $this->order_total = $value;
    }

    public function getOrderTotal($format=False)
    {
        return ($format) ? number_format($this->order_total, 2, ".", "") :
                                         $this->order_total;
    }

    public function setShippingPrice($value)
    {
        $this->shipping_price = $value;
    }

    public function getShippingPrice()
    {
        return $this->shipping_price;
    }

    public function getPaymentFormType()
    {
        if (strpos($this->order_params["form_url"], "/order/inline") == True)
            return "order";
        else
            return "create";
    }

    public function setUseTaxes()
    {
        $this->use_taxes = True;
    }

    public function getUseTaxes()
    {
        return $this->use_taxes;
    }

    public function setUseDelivery()
    {
        $this->use_delivery = True;
    }

    public function getUseDelivery()
    {
        return $this->use_delivery;
    }

    public function recalculateTaxes($item_pos)
    {
        //recalculate taxes
        switch($this->fiscal_cart[$item_pos]['tax']) {
            case "vat10":
                $this->fiscal_cart[$item_pos]['tax_sum'] = round((float)
                    (($this->fiscal_cart[$item_pos]['sum']/110)*10), 2);
                break;
            case "vat18":
                $this->fiscal_cart[$item_pos]['tax_sum'] = round((float)
                    (($this->fiscal_cart[$item_pos]['sum']/118)*18), 2);
                break;
        }
    }

    public function recalculateFiscalCart($mod_value, $with_exclude_index)
    {
        $exclude_index = ($with_exclude_index) ? $this->delivery_index : -1;
        //iterate fiscal cart without shipping
        for ($pos=0; $pos<count($this->getFiscalCart()); $pos++) {
            if ($pos != $exclude_index) {
                $this->fiscal_cart[$pos]["sum"] *= $mod_value;
                $this->fiscal_cart[$pos]["price"] = $this->fiscal_cart[$pos]["sum"]/$this->fiscal_cart[$pos]["quantity"];
                //formatting
                $this->fiscal_cart[$pos]["price"] = number_format($this->fiscal_cart[$pos]["price"], 3, ".", "");
                $this->fiscal_cart[$pos]["sum"] = number_format($this->fiscal_cart[$pos]["sum"], 2, ".", "");
                //recalculate taxes
                $this->recalculateTaxes($pos);
            }
        }
    }

    public function setTaxes($sum, $tax_rate)
    {
        $taxes = array("tax" => "none", "tax_sum" => 0);
        switch(number_format($tax_rate, 0, ".", "")) {
            case 0:
                $taxes["tax"] = "vat0";
                $taxes["tax_sum"] = number_format(0, 2, ".", "");
                break;
            case 10:
                $taxes["tax"] = "vat10";
                $taxes["tax_sum"] = round((float)(($sum/110)*10), 2);
                break;
            case 18:
                $taxes["tax"] = "vat18";
                $taxes["tax_sum"] = round((float)(($sum/118)*18), 2);
                break;
            case 20:
                $taxes["tax"] = "vat20";
                $taxes["tax_sum"] = round((float)(($sum/120)*20), 2);
                break;
        }
        return $taxes;
    }

    public function checkDeliveryIncluded($delivery_price, $delivery_name) {
        $index = 0;
        foreach ($this->getFiscalCart() as $item) {
            if ($item["name"] == $delivery_name
                && $item["price"] == $delivery_price
                && $item["quantity"] == 1)
                $this->delivery_included = true;
                $this->delivery_index = $index;
            $index++;
        }
    }
    
    public function isDeliveryIncluded() {
        return $this->delivery_included;
    }

    public function setDeliveryIndex($index_value) {
        $this->delivery_index = $index_value;
    }

    public function getDeliveryIndex() {
        return $this->delivery_index;
    }

    public function getFiscalCartSum($delivery_included) {
        $fiscal_cart_sum = 0;
        $index = 0;
        foreach ($this->getFiscalCart() as $item) {
            if (!$delivery_included && $index == $this->delivery_index)
                continue;
            $fiscal_cart_sum += ((float) $item["price"] * (int) $item["quantity"]);
            $index++;
        }
        return $fiscal_cart_sum;
    }

    public function showDebugInfo($obj_to_debug)
    {
        echo "<pre>";
        var_dump($obj_to_debug);
        echo "</pre>";
    }

}
